<?php

/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 *
 * @category    IyzicoCheckout
 * @package     IyzicoCheckout_IyzicoCheckout
 * @copyright   Copyright (c) 2013 Iyzico  (http://www.iyzico.com/)
 */

/**
 * Iyzico helper
 *
 * @author Iyzico
 */
class IyzicoCheckout_IyzicoCheckout_Helper_Data extends Mage_Core_Helper_Abstract
{

    /**
     * Retrieve quote object
     *
     * @return Mage_Sales_Model_Quote
     */
    public function getQuote()
    {
        return Mage::getModel('checkout/session')->getQuote();
    }

    /**
     * Retrieve order object
     *
     * @param int $id
     * @return Mage_Sales_Model_Order
     */
    public function getOrder($id)
    {
        return Mage::getSingleton('sales/order')->load($id);
    }

    /**
     * Retrieve name data
     *
     * @param Mage_Sales_Model_Order $order
     * @return array
     */
    public function getUserNameData(Mage_Sales_Model_Order $order)
    {
        $billingParamsObj = $order->getBillingAddress();
        if (!empty($billingParamsObj)) {
            $firstName = $billingParamsObj->getFirstname();
            $lastName = $billingParamsObj->getLastname();
            $company = $billingParamsObj->getCompany();
        } else {
            $firstName = '';
            $lastName = '';
            $company = '';
        }
        $data = array(
            'first_name' => $firstName,
            'last_name' => $lastName,
            'company' => $company,
        );

        return $data;
    }

    /**
     * Retrieve address data
     *
     * @param Mage_Sales_Model_Order $order
     * @return array
     */
    public function getUserAddressData(Mage_Sales_Model_Order $order)
    {
        $shippingParamObj = $order->getShippingAddress();
        $billingParamObj = $order->getBillingAddress();
        $data = array();
        if (!empty($shippingParamObj)) {
            $countryNameShipping = '';
            $shippingCountryCode = $shippingParamObj->getCountryId();

            if (!empty($shippingCountryCode)) {
                $country = Mage::getModel('directory/country')->loadByCode($shippingCountryCode);
                $countryNameShipping = $country->getName();
            }

            $data['customer_shipping_address_zip'] = $shippingParamObj->getPostcode();
            $data['customer_shipping_address_city'] = $shippingParamObj->getCity();
            $data['customer_shipping_address_street'] = $shippingParamObj->getStreetFull();
            $data['customer_shipping_address_state'] = $shippingParamObj->getRegion();
            $data['customer_shipping_address_country'] = $countryNameShipping;
        }

        if (!empty($billingParamObj)) {
            $billingCountryCode = $billingParamObj->getCountryId();
            $countryNameBilling = '';

            if (!empty($billingCountryCode)) {
                $country = Mage::getModel('directory/country')->loadByCode($billingCountryCode);
                $countryNameBilling = $country->getName();
            }

            $data['zip'] = $billingParamObj->getPostcode();
            $data['street'] = $billingParamObj->getStreetFull();
            $data['city'] = $billingParamObj->getCity();
            $data['state'] = $billingParamObj->getRegion();
            $data['country_billing'] = $countryNameBilling;
        }

        return $data;
    }

    /**
     * Retrieve contact data
     *
     * @param Mage_Sales_Model_Order $order
     * @return array
     */
    public function getUserContactData(Mage_Sales_Model_Order $order)
    {
        $billingParams = $order->getBillingAddress();
        $phone = '';
        if (!empty($billingParams)) {
            $phone = $billingParams->getTelephone();
        }
        $data = array(
            'email' => $order->getCustomerEmail(),
            'phone' => $phone,
            'ip' => Mage::helper('core/http')->getRemoteAddr(false)
        );

        return $data;
    }

    /**
     * Retrieve Basket data
     *
     * @param Mage_Sales_Model_Order $order
     * @return array
     */
    public function getPaymentBasketData(Mage_Sales_Model_Order $order)
    {
        $basket = array(
            'amount' => $order->getGrandTotal(),
            'currency' => $order->getOrderCurrencyCode()
        );

        return $basket;
    }

    /**
     * Retrieve method code for config loading
     *
     * @param Mage_Sales_Model_Order $order
     * @return string
     */
    public function getMethodCode(Mage_Sales_Model_Order $order)
    {
        return str_replace('iyzicocheckout', 'method', $order->getPayment()->getMethod());
    }

    /**
     * Retrieve the locale code in iso (2 chars)
     *
     * @return string
     */
    public function getLocaleIsoCode()
    {
        return substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2);
    }

    public function userInvoice(Mage_Sales_Model_Order $order)
    {
        $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
        $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);
        $invoice->register();
        $invoice->getOrder()->setCustomerNoteNotify(false);
        $invoice->getOrder()->setIsInProcess(true);
        $transactionSave = Mage::getModel('core/resource_transaction')
                ->addObject($invoice)
                ->addObject($invoice->getOrder());
        $transactionSave->save();
        $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true)->save();
    }

}
