<?php

/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 *
 * @category    IyzicoCheckout
 * @package     IyzicoCheckout_IyzicoCheckout
 * @copyright   Copyright (c) 2013 Iyzico  (http://www.iyzico.com/)
 */

/**
 * IYZICO coummnication model
 *
 * @author Iyzico
 */
class IyzicoCheckout_IyzicoCheckout_Model_IyzicoCheckout_Communication extends Zend_Http_Client
{

    /**
     * IYZICO Web service url
     *
     * @var string
     */
    private $_webServiceUrl = 'https://api.iyzico.com/v2/create';

    /**
     * Testmode flag
     *
     * @var boolean
     */
    private $_testMode = false;

    /**
     * HTTP config array
     *
     * @var array
     */
    protected $config = array(
        'strictredirects' => false,
        'adapter' => 'Zend_Http_Client_Adapter_Curl',
        'curloptions' => array(
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_POST => 1,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_HTTPHEADER => array(
                "Content-Type:  application/x-www-form-urlencode; charset=UTF-8",
                "Accept: */*",
                "Cache-Control: no-cache",
                "Pragma: no-cache",
                "Connection: keep-alive"
            )
        ),
        'maxredirects' => 5,
        'useragent' => 'Zend_Http_Client',
        'timeout' => 10,
        'httpversion' => self::HTTP_1,
        'keepalive' => false,
        'storeresponse' => true,
        'strict' => true,
        'output_stream' => false,
        'encodecookies' => true,
        'rfc3986_strict' => false
    );

    /**
     * Constructs Object with Server URL and Configs
     * @todo see review document
     * @param array
     */
    public function __construct(array $sandboxMode = null)
    {
        if (!empty($sandboxMode)) {
            $this->setTestMode($sandboxMode[0]);
        }

        parent::__construct($this->getIyzicoServerUrl(), $this->getConfig());
    }

    /**
     * Sets the Testmode to on/off
     *
     * @param boolean
     */
    public function setTestMode($testMode = false)
    {
        $this->_testMode = $testMode;
    }

    /**
     * Get the Testmode on or off
     *
     * @return boolean
     */
    public function getTestMode()
    {
        return $this->_testMode;
    }

    /**
     * Returns the Server URL depending 
     *
     * @return boolean
     */
    public function getIyzicoServerUrl()
    {
        return $this->_webServiceUrl;
    }

    /**
     * Returns the Config of the Adapter
     *
     * @return array
     */
    public function getConfig()
    {
        return $this->config;
    }

}
