<?php

/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 *
 * @category    IyzicoCheckout
 * @package     IyzicoCheckout_IyzicoCheckout
 * @copyright   Copyright (c) 2013 Iyzico  (http://www.iyzico.com/)
 */

/**
 * IYZICO request model
 *
 * @author Iyzico
 */
class IyzicoCheckout_IyzicoCheckout_Model_IyzicoCheckout_Request extends Mage_Core_Model_Abstract
{

    /**
     * Client object
     *
     */
    protected $_client;

    /**
     *
     * @var Mage_Sales_Model_Order
     */
    private $_order;

    /**
     * Construct
     */
    public function __construct(array $params)
    {
        $this->_setOrder($params[0]);
        $this->_client = Mage::getSingleton('iyzicocheckout/iyzicoCheckout_communication', array($this->_getPaymentObject()->isTestserver()));
    }

    /**
     * Retrieve the HTTP client object
     *
     * @return Iyzico_Iyzico_Model_Iyzico_Communication
     */
    protected function _getClient()
    {
        if ($this->_client == null) {
            throw new Exception('Request client is not initialized!', -1);
        }

        return $this->_client;
    }

    /**
     * Set payment for the IYZICO post call
     *
     * @throws Exception
     */
    protected function _setPayment()
    {
        try {
            $this->_setParameterPost('PAYMENT.CODE', $this->_getPaymentObject()->getPaymentCode());
        } catch (Exception $e) {
            throw new Exception($e->getMessage(), $e->getCode());
        }
    }

    /**
     * Set the transaction id for the IYZICO post call
     * @todo see review document
     */
    protected function _setTransactionId($prefix)
    {
        $orderId = '';
        if (empty($prefix)) {
            $prefix = 'magento_MBCR';
        }
        if ($this->_getOrder() instanceof Mage_Sales_Model_Order) {
            $orderId = $this->_getOrder()->getIncrementId();
        } else {
            $orderId = $this->_getOrder()->getReservedOrderId();
        }
        $this->_setParameterPost('external_id', str_replace(".", "", uniqid($prefix . '_', true)) . '_' . $orderId);
    }

    /**
     * Set the IYZICO post call transaction mode
     * @todo see review document
     */
    protected function _setTransactionMode()
    {
        $mode = '';
        if ($this->_getPaymentObject()->isTestmode()) {
            $mode = $this->getTestmode();
        } else {
            $mode = 'live';
        }

        $this->_setParameterPost('mode', $mode);
    }

    protected function getTestmode()
    {
        $code = $this->_getPaymentObject()->getCode();
        switch ($code) {
            case 'iyzicocheckout_creditcard':
                return 'test';
                break;
            default:
                return 'test';
                break;
        }
    }

    /**
     * Set the IYZICO post call credentials
     *
     * @throws Exception
     */
    protected function _setCredentials()
    {
        $credentials = $this->_getPaymentObject()->getCredentials();
        if (!empty($credentials)) {
            $this->_setParameterPost('api_id', $credentials['api_id']);
            $this->_setParameterPost('secret', $credentials['secret']);
            $this->_setTransactionId($credentials['prefix']);
            $this->_setParameterPost('installment', $credentials['installment']);
        } else {
            throw new Exception('The credentials are empty!', -2);
        }
    }

    /**
     * Set the customer data for the iyzico call
     *
     * @throws Exception
     */
    protected function _setCustomer()
    {
        try {
            $this->_setAddress();
            $this->_setName();
            $this->_setContact();
        } catch (Exception $e) {
            throw new Exception($e->getMessage(), $e->getCode());
        }
    }

    /**
     * Set the name data for the IYZICO call
     *
     * @throws Exception
     */
    private function _setName()
    {
        $data = Mage::helper('iyzicocheckout')->getUserNameData($this->_getOrder());
        if (!empty($data)) {
            $this->_setParameterPost('customer_first_name', $data['first_name']);
            $this->_setParameterPost('customer_last_name', $data['last_name']);
            $this->_setParameterPost('customer_presentation_usage', 'iyzicocheckout' . uniqid());
            $this->_setParameterPost('customer_name_company', $data['company']);
        } else {
            throw new Exception('The name data are empty!', -3);
        }
    }

    /**
     * Set the name data for the IYZICO call
     *
     * @throws Exception
     */
    private function _setAddress()
    {
        $data = Mage::helper('iyzicocheckout')->getUserAddressData($this->_getOrder());
        if (!empty($data)) {
            if (!empty($data['street'])) {
                $this->_setParameterPost('customer_billing_address_line_1', $data['street']);
            }
            if (!empty($data['zip'])) {
                $this->_setParameterPost('customer_billing_address_zip', $data['zip']);
            }
            if (!empty($data['city'])) {
                $this->_setParameterPost('customer_billing_address_city', $data['city']);
            }
            if (!empty($data['state'])) {
                $this->_setParameterPost('customer_billing_address_state', $data['state']);
            }
            if (!empty($data['country_billing'])) {
                $this->_setParameterPost('customer_billing_address_country', $data['country_billing']);
            }
            if (!empty($data['customer_shipping_address_state'])) {
                $this->_setParameterPost('customer_shipping_address_state', $data['customer_shipping_address_state']);
            }
            if (!empty($data['customer_shipping_address_country'])) {
                $this->_setParameterPost('customer_shipping_address_country', $data['customer_shipping_address_country']);
            }
            if (!empty($data['customer_shipping_address_street'])) {
                $this->_setParameterPost('customer_shipping_address_line_1', $data['customer_shipping_address_street']);
            }
            if (!empty($data['customer_shipping_address_city'])) {
                $this->_setParameterPost('customer_shipping_address_city', $data['customer_shipping_address_city']);
            }
            if (!empty($data['customer_shipping_address_zip'])) {
                $this->_setParameterPost('customer_shipping_address_zip', $data['customer_shipping_address_zip']);
            }
        } else {
            throw new Exception('The address data are empty!', -3);
        }
    }

    /**
     * Set the name data for the IYZICO call
     *
     * @throws Exception
     */
    private function _setContact()
    {
        $data = Mage::helper('iyzicocheckout')->getUserContactData($this->_getOrder());
        if (!empty($data)) {
            $this->_setParameterPost('customer_contact_email', $data['email']);
            $this->_setParameterPost('customer_contact_phone', $data['phone']);
            $this->_setParameterPost('customer_contact_ip', $data['ip']);
        } else {
            throw new Exception('The contact data are empty!', -3);
        }
    }

    /**
     * Set the basket data for the IYZICO call
     */
    protected function _setRequestData()
    {
        $siteLang = explode('_', Mage::app()->getLocale()->getLocaleCode());
        $basket = Mage::helper('iyzicocheckout')->getPaymentBasketData($this->_getOrder());
        if (!empty($basket)) {
            $this->_setParameterPost('amount', $this->formateLessAmountConverter($basket['amount']));
            $this->_setParameterPost('currency', $basket['currency']);
            $this->_setParameterPost('type', 'CC.DB');
            $this->_setParameterPost('return_url', $this->removeSIDqueryStrVar() . 'iyzicocheckout/response/handleIyzicoPostResponse');
            $this->_setParameterPost('customer_language', $siteLang[0]);
        } else {
            throw new Exception('The basket data are empty!', -4);
        }
    }

    protected function _setItemsArr()
    {
        $cartItems = Mage::getModel("checkout/cart")->getItems();
        $index = 1;
        if (!empty($cartItems)) {
            foreach ($cartItems as $item) {
                $itemQty = $item->getQty();
                $itemPrice = $item->getProduct()->getPrice();
                $this->_setParameterPost('item_id_' . $index, $item->getProductId());
                $this->_setParameterPost('item_name_' . $index, $item->getProduct()->getName());
                $this->_setParameterPost('item_unit_quantity_' . $index, $itemQty);
                $this->_setParameterPost('item_unit_amount_' . $index, $itemPrice);
                $index ++;
            }
        }
    }

    protected function _setBasket()
    {
        $basket = Mage::helper('iyzicocheckout')->getPaymentBasketData($this->_getOrder());
        if (!empty($basket)) {
            $this->_setParameterPost('PAYMENT.TYPE', 'DB');
        } else {
            throw new Exception('The basket data are empty!', -4);
        }
    }

    public function removeSIDqueryStrVar()
    {
        return strtok(Mage::getUrl('', array('_secure' => true)), '?');
    }

    /**
     * Calls the IYZICO page call
     *
     * @return array
     * @throws Exception
     */
    public function request()
    {
        try {
            $this->_getClient()->resetParameters();
            $this->_setCredentials();
            $this->_setRequestData();
            $this->_setTransactionMode();
            $this->_setCustomer();
            $this->_setItemsArr();

            $response = $this->_getClient()->request('POST');
            return $response->getBody();
        } catch (Exception $e) {
            throw new Exception($e->getMessage(), $e->getCode());
        }
    }

    public function formateLessAmountConverter($amount)
    {
        if (is_numeric($amount)) {
            $formattedAmount = str_replace(',', '.', $amount);
            return $formattedAmount * 100;
        } else {
            return false;
        }
    }

    /**
     * Parse the query string into a array
     *
     * @param string $string
     * @return array
     */
    protected function _parseResponse($string)
    {
        $data = array();
        parse_str($string, $data);
        return $data;
    }

    /**
     * Retrieve order instance
     *
     * @return Mage_Sales_Model_Order
     */
    private function _getOrder()
    {
        return $this->_order;
    }

    /**
     * Set order instance
     *
     * @param Mage_Sales_Model_Order $order
     */
    private function _setOrder(Mage_Sales_Model_Order $order)
    {
        $this->_order = $order;
    }

    /**
     * Retrieve method instance
     *
     * @return type
     */
    private function _getPaymentObject()
    {
        return $this->_getOrder()->getPayment()->getMethodInstance();
    }

    /**
     * Set's only a non empty parameter
     *
     * @param string $index
     * @param string $value
     */
    private function _setParameterPost($index, $value)
    {
        if (!empty($value) || $value === '0') {
            $this->_getClient()->setParameterPost($index, $value);
        }
    }

}
