<?php

/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 *
 * @category    IyzicoCheckout
 * @package     IyzicoCheckout_IyzicoCheckout
 * @copyright   Copyright (c) 2013 Iyzico  (http://www.iyzico.com/)
 */

/**
 * Abstract payment model
 *
 * @author Iyzico
 */
abstract class IyzicoCheckout_IyzicoCheckout_Model_Method_Abstract extends Mage_Payment_Model_Method_Abstract
{

    /**
     * Is method a gateaway
     *
     * @var boolean

     */
    protected $_isGateway = true;

    /**
     * Can this method use for checkout
     *
     * @var boolean
     */
    protected $_canUseCheckout = true;

    /**
     * Can this method use for multishipping
     *
     * @var boolean
     */
    protected $_canUseForMultishipping = false;

    /**
     * Is a initalize needed
     *
     * @var boolean
     */
    protected $_isInitializeNeeded = false;

    /**
     * Method code for iyzico call
     *
     * @var type
     */
    protected $_methodCode = '';

    /**
     * Payment Title
     *
     * @var type
     */
    protected $_methodTitle = '';

    /**
     * Payment code for iyzico call
     * @var string
     */
    protected $_paymentCode = 'DB';

    /**
     * FRONTEND.SUBTYPE for iyzico call
     *
     * @var string
     */
    protected $_subtype = '';

    /**
     * Magento method code
     *
     * @var string
     */
    protected $_code = 'iyzicocheckout_abstract';

    /**
     * Collect Data on IYZICO
     *
     * @var string
     */
    protected $_collectData = '';

    /**
     * Redirect or iFrame
     * @var type
     */
    protected $_implementation = 'iframe';
    protected $_canCapture = true;

    /**
     * Retrieve the credentials
     *
     * @return array
     */
    public function getCredentials()
    {
        if (Mage::getStoreConfig('payment/' . $this->getCode() . '/testmode', $this->getOrder()->getStoreId()) == 1) {
            $credentials = array(
                'api_id' => Mage::getStoreConfig('payment/' . $this->getCode() . '/test_api_id', $this->getOrder()->getStoreId()),
                'secret' => Mage::getStoreConfig('payment/' . $this->getCode() . '/test_secret_key', $this->getOrder()->getStoreId()),
                'prefix' => Mage::getStoreConfig('payment/' . $this->getCode() . '/prefix', $this->getOrder()->getStoreId()),
                'installment' => Mage::getStoreConfig('payment/' . $this->getCode() . '/installment', $this->getOrder()->getStoreId()),
            );
        } else {
            $credentials = array(
                'api_id' => Mage::getStoreConfig('payment/' . $this->getCode() . '/live_api_id', $this->getOrder()->getStoreId()),
                'secret' => Mage::getStoreConfig('payment/' . $this->getCode() . '/live_secret_key', $this->getOrder()->getStoreId()),
                'prefix' => Mage::getStoreConfig('payment/' . $this->getCode() . '/prefix', $this->getOrder()->getStoreId()),
                'installment' => Mage::getStoreConfig('payment/' . $this->getCode() . '/installment', $this->getOrder()->getStoreId()),
            );
        }
        return $credentials;
    }

    /**
     * Retrieve testmode flag
     *
     * @return boolean
     */
    public function isTestmode()
    {
        return Mage::getStoreConfig('payment/' . $this->getCode() . '/testmode', $this->getOrder()->getStoreId());
    }

    public function isTestserver()
    {
        return Mage::getStoreConfig('payment/' . $this->getCode() . '/testmode', $this->getOrder()->getStoreId());
    }

    /**
     * Return Quote or Order Object depending what the Payment is
     *
     * @return Mage_Sales_Model_Order
     */
    public function getOrder()
    {
        $paymentInfo = $this->getInfoInstance();

        if ($paymentInfo instanceof Mage_Sales_Model_Order_Payment) {
            return $paymentInfo->getOrder();
        }

        return $paymentInfo->getQuote();
    }

    /**
     * Retrieve the order place URL
     *
     * @return string
     */
    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getSingleton('customer/session')->getRedirectUrl();
    }

    /**
     * Calling the IYZICO Post call
     *
     * @param Varien_Object $payment
     * @param type $amount
     */
    public function authorize(Varien_Object $payment, $amount)
    {
        try {
            $response = Mage::getSingleton('iyzicocheckout/iyzicoCheckout_request', array($this->getOrder()))->request();
            $paymentTokenResp = json_decode($response);
            if (is_object($paymentTokenResp)) {
                if (property_exists($paymentTokenResp, 'code_snippet')) {
                    $payment->setAdditionalInformation('iyzico_transaction_code', $this->getIyzicoTransactionCode());
                    if ($this->_getImplementation() == 'iframe') {
                        $this->_iframe($paymentTokenResp);
                    } else {
                        $this->_redirect($response);
                    }
                } else if (property_exists($paymentTokenResp, 'error_message')) {
                    Mage::throwException(Mage::helper('iyzicocheckout')->__($paymentTokenResp->response->error_message));
                } else {
                    Mage::throwException(Mage::helper('iyzicocheckout')->__($paymentTokenResp->response->error_message));
                }
            } else {
                Mage::throwException(Mage::helper('iyzicocheckout')->__('Error before redirect'));
            }
        } catch (Exception $e) {
            Mage::getSingleton('checkout/session')->setGotoSection('payment');
            Mage::throwException($e->getMessage());
        }

        return $this;
    }

    public function capture(Varien_Object $payment, $amount)
    {
        $response = Mage::getSingleton('iyzicocheckout/iyzicocheckout_request', array($this->getOrder()))->request();
        if ($response['POST_VALIDATION'] == 'ACK') {
            $payment->setStatus('APPROVED')
                    ->setTransactionId($payment->getAdditionalInformation('IDENTIFICATION_REFERENCEID'))
                    ->setIsTransactionClosed(1)->save();
        } else {
            Mage::throwException(Mage::helper('iyzicocheckout')->__('Error while capturing'));
        }

        return $this;
    }

    /**
     * Retrieve IYZICO payment code
     *
     * @return string
     */
    public function getPaymentCode()
    {
        return $this->_methodCode . "." . $this->getIyzicoTransactionCode();
    }

    public function getIyzicoTransactionCode()
    {
        return $this->_paymentCode;
    }

    /**
     * Retrieve IYZICO method
     *
     * @return string
     */
    public function getMethod()
    {
        return $this->_methodCode;
    }

    /**
     * Retrieve IYZICO subtype
     *
     * @return string
     */
    public function getSubtype()
    {
        return $this->_subtype;
    }

    /**
     * Retrieve wether to collect Data on IYZICO or not
     *
     * @return string
     */
    public function getCollectData()
    {
        return $this->_collectData;
    }

    /**
     * Returns Payment Title
     *
     * @return string
     */
    public function getTitle()
    {
        return Mage::helper('iyzicocheckout')->__($this->_methodTitle);
    }

    /**
     * Set the iframe Url
     *
     * @param array $response
     */
    protected function _iframe($response)
    {
        Mage::getSingleton('customer/session')->setIframeUrl(Mage::getUrl('iyzicocheckout/response/handleIyzicoPostResponse/'));
        Mage::getSingleton('customer/session')->setIframeFlag(true);
        Mage::getSingleton('customer/session')->setToken($response->transaction_token);
        Mage::getSingleton('customer/session')->setCodeSnippet($response->code_snippet);
        Mage::getSingleton('customer/session')->setMode($this->isTestmode());
        Mage::getSingleton('customer/session')->setRedirectUrl(Mage::getUrl('iyzicocheckout/response/renderIframe/'));
    }

    /**
     * Set the redirect Url
     *
     * @param array $response
     */
    protected function _redirect($response)
    {
        Mage::getSingleton('customer/session')->setIframeFlag(false);
        Mage::getSingleton('customer/session')->setRedirectUrl($response['FRONTEND_REDIRECT_URL']);
    }

    /**
     * Retrieve implementation method
     *
     * @return string
     */
    protected function _getImplementation()
    {
        return $this->_implementation;
    }

}
