<?php

/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 *
 * @category    IyzicoCheckout
 * @package     IyzicoCheckout_IyzicoCheckout
 * @copyright   Copyright (c) 2013 Iyzico  (http://www.iyzico.com/)
 */

/**
 * Response controller
 *
 * @author Iyzico
 */
class IyzicoCheckout_IyzicoCheckout_ResponseController extends Mage_Core_Controller_Front_Action
{

    /**
     * Construct
     */
    protected $_getStatusUrl = 'https://api.iyzico.com/getStatus';

    public function _construct()
    {
        parent::_construct();
    }

    /**
     * Handles the IYZICO post response after the redirect to the shop
     *
     */
    public function handleIyzicoPostResponseAction()
    {
        $response = "";
        $postDataResponse = Mage::app()->getRequest()->getPost();
        if (!empty($postDataResponse['json'])) {
            $response = $postDataResponse['json'];
            $order = Mage::getSingleton('sales/order');
            $url = $this->_getPostResponseActionUrl($order, $response);
            $this->_redirectUrl($url);
        } else {
            return $this->_redirect('checkout/cart');
        }
    }

    /**
     * Retrieve redirect url for the IYZICO
     *
     * @param Mage_Sales_Model_Order $order
     * @return string
     */
    private function _getPostResponseActionUrl(Mage_Sales_Model_Order $order, $responseOriginal)
    {
        $pageName = '';
        $params = array();
        $response = json_decode($responseOriginal);
        $prefix = $this->findPrefixfromSore();
        $cartItem = Mage::helper('checkout/cart')->getCart()->getItemsCount();
        $externalId = isset($response->transaction->external_id) ? $response->transaction->external_id : '';

        if (empty($prefix)) {
            $prefix = 'magento_MBCR';
        }

        if (!empty($externalId) && !empty($cartItem)) {
            $dataArr = explode('_', $response->transaction->external_id);
            $orderId = $dataArr[count($dataArr) - 1];
            $order->loadByIncrementId($orderId);
            $orderState = $order->getStatus();
        } else {
            return $this->_setErrorRedirect("Invalid Order");
        }

        if ($orderState == 'pending_payment') {

            if ($response->response->state == 'success') {

                $getStatusResponseFromIyzico = $this->getStatusCallToIyzico($this->_getStatusUrl . '?token=' . $response->transaction_token);
                $getStatusResponse = json_decode($getStatusResponseFromIyzico);

                if ($getStatusResponse->response->state == 'success' && $getStatusResponse->transaction->state == 'paid') {

                    $order->getPayment()->setAdditionalInformation(
                            'IDENTIFICATION_REFERENCEID', $response->transaction_token
                    );
                    $iyzicoAmount = !empty($response->transaction->amount) ? $response->transaction->amount : 0;
                    $installmentCount = !empty($response->installment->count) ? $response->installment->count : 0;

                    if ($installmentCount > 0) {
                        $granTotalCart = $order->getGrandTotal();
                        $installmentFee = $iyzicoAmount - $granTotalCart;
                        $order->setGrandTotal($iyzicoAmount);
                        $order->setIyzicoinstallmentAmount($installmentFee);
                        $order->setNumberOfInstallment($installmentCount);
                    }

                    $order->save();
                    if ($order->getPayment()->getAdditionalInformation('iyzico_transaction_code') == 'DB') {

                        Mage::helper('iyzicocheckout')->userInvoice($order);
                    } else {
                        $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true)->save();
                    }

                    Mage::getModel('sales/quote')->load($order->getQuoteId())->setIsActive(false)->save();
                    $pageName = 'checkout/onepage/success/';
                    // send order email
                    $order->sendNewOrderEmail();
                    $params['_secure'] = true;
                    return Mage::getUrl($pageName, $params);
                } else {
                    $this->writeResponseToFile('Invalid Payment Information During get Status Call : ' . $orderId);
                    return $this->_setErrorRedirect('Invalid Payment Information');
                }
            } else {

                if ($response->response->state) {
                    $message = $response->response->error_message;
                } else {
                    $message = "Error before redirect";
                }
                /* write the log into log file */
                $this->writeResponseToFile($response);
                if (!empty($response->transaction->external_id)) {
                    $order->cancel()->save();
                }
                return $this->_setErrorRedirect($message);
            }
        } else {
             $this->writeResponseToFile('Invalid Order : Order Status '. $orderState . 'Order ID '. $orderId);
             return $this->_setErrorRedirect("Invalid Order or Order is already placed");
        }
    }

    /**
     * Add error and redirect to the cart
     */
    public function redirectToErrorAction()
    {
        $message = $this->myUrlDecode($this->getRequest()->getParam('message'));
        if (!empty($message)) {
            Mage::getSingleton('core/session')->addError($message);
        }

        $this->_redirect('checkout/cart/');
    }

    public function myUrlDecode($val)
    {
        return str_replace("%2f", "/", urldecode($val));
    }

    /**
     * Render the iFrame page
     */
    public function renderIframeAction()
    {
        $this->loadLayout();
        $block = $this->getLayout()->createBlock('iyzicocheckout/payment_iframe');

        $this->getLayout()->getBlock('content')->append($block);
        $this->renderLayout();
    }

    /*
     * @write response in file 
     */

    public function writeResponseToFile($response)
    {
        $iyzicoCheckout = Mage::getBaseDir('var') . DIRECTORY_SEPARATOR . 'log' . DIRECTORY_SEPARATOR . 'iyzicocheckout';
        if (!is_dir($iyzicoCheckout)) {
            mkdir($iyzicoCheckout, 0777, true);
        }
        $fileName = $iyzicoCheckout . DIRECTORY_SEPARATOR . date('Y-m-d') . '.txt';
        $fp = fopen($fileName, 'a') or die("can't open file");
        $currentContent .= "\n------------------------------------------------------------------------------------------------------------------\n"
                . date('Y-m-d H:i:s') . "\nRemote Ip: " . $_SERVER['REMOTE_ADDR']
                . "\n" . json_encode($response) . "\n";
        fwrite($fp, $currentContent);
        fclose($fp);
    }

    public function isTestMode()
    {
        if (Mage::getStoreConfig('payment/iyzicocheckout_creditcard/testmode', Mage::app()->getStore()->getStoreId()) == 1) {
            return true;
        } else {
            return false;
        }
    }

    public function findPrefixfromSore()
    {
        $prefix = Mage::getStoreConfig('payment/iyzicocheckout_creditcard/prefix', Mage::app()->getStore()->getStoreId());
        return $prefix;
    }

    public function getShopingCartAmount()
    {
        $quote = Mage::getSingleton('checkout/cart')->getQuote();
        $grandTotal = $quote->getGrandTotal();
        return $grandTotal;
    }

    public function getStatusCallToIyzico($url)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);

        $curlResponse = curl_exec($ch);
        curl_close($ch);
        return $curlResponse;
    }

    private function _setErrorRedirect($message)
    {
        Mage::getSingleton('core/session')->setPiIyzicoPaymentError($this->__($message));
        $pageName = 'iyzicocheckout/response/redirectToError/';
        $params = array('message' => $this->__($message));
        $params['_secure'] = true;
        return Mage::getUrl($pageName, $params);
    }

}
